<?php
namespace FiloBlu\StockaFix\Controller\Adminhtml\Report;

use FiloBlu\StockaFix\Model\Report;
use FiloBlu\StockaFix\Helper\MailHelper;
use Magento\Framework\App\ResourceConnection;
use Zend_Db;

/**
 * Class SendAction
 * @package FiloBlu\StockaFix\Controller\Adminhtml\Report
 */
class Send extends \Magento\Backend\App\Action
{
    /**
     * @var MailHelper
     */
    private $mailHelper;

    /**
     * @var ResourceConnection
     */
    private $resourceConnection;


    /**
     * @param \Magento\Backend\App\Action\Context $context
     * @param MailHelper $mailHelper
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        MailHelper $mailHelper,
        ResourceConnection $resourceConnection
    )
    {
        $this->mailHelper = $mailHelper;
        $this->resourceConnection = $resourceConnection;
        parent::__construct($context);
    }


    /**
     * @return mixed
     */
    public function execute()
    {
        $id = $this->getRequest()->getParam('report_id');

        if (!($report = $this->_objectManager->create(Report::class)->load($id))) {
            $this->messageManager->addErrorMessage(__('Unable to proceed. Report not found.'));
            $resultRedirect = $this->resultRedirectFactory->create();
            return $resultRedirect->setPath('*/*/edit', ['_current' => true, 'id' => $id]);
        }

        try{
            $reportOutput = $this->resourceConnection->getConnection()->fetchAll($report->getSql(), [], Zend_Db::FETCH_ASSOC);
            $count = count($reportOutput);
            if($count > 0) {
                $csv = $this->mailHelper->produceReport($report, $reportOutput);

                $reportFilename = null;
                if($report->getData('schedule_filename')) {
                    $reportFilename = str_replace('%d', date('Y_m_d'), $report->getData('schedule_filename'));
                }

                $to = explode(',', $report->getReceiverEmails());
                $this->mailHelper->sendReportMail($to, [$csv], ['report_name' => $report->getName(), 'report_count' => $count, 'report_filename' => $reportFilename]);

                $this->messageManager->addSuccessMessage(__('Your report has been sended !'));
            }
        } catch (\Exception $e) {
            $this->messageManager->addErrorMessage(__('Error while trying to send report: '.$e->getMessage()));
            $resultRedirect = $this->resultRedirectFactory->create();
            return $resultRedirect->setPath('*/*/edit', ['_current' => true, 'id' => $id]);
        }

        $resultRedirect = $this->resultRedirectFactory->create();
        return $resultRedirect->setPath('*/*/edit', ['_current' => true, 'id' => $id]);
    }
}
